import type { LanguageInput, ThemedToken, ThemeInput, ThemeRegistrationAny, TokensResult } from '@shikijs/types'
import type { ScrollViewProps, StyleProp, TextStyle, ViewStyle } from 'react-native'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { Platform, ScrollView, StyleSheet, Text, View } from 'react-native'

const FONT_FAMILY = Platform.select({
  ios: 'Menlo',
  android: 'monospace',
  default: 'monospace',
})

interface SyntaxHighlighterProps {
  text: string
  language?: LanguageInput
  languageId?: string
  theme: ThemeRegistrationAny
  fontSize?: number
  scrollViewProps?: ScrollViewProps
  textStyle?: StyleProp<TextStyle>
  containerStyle?: StyleProp<ViewStyle>
}

const engine = createJavaScriptRegexEngine({
  forgiving: true,
  simulation: true,
  cache: new Map(),
})

async function createHighlighter({ langs, themes }: { langs: LanguageInput[], themes: ThemeInput[] }) {
  if (!langs?.length) {
    throw new Error('Please provide languages when creating a highlighter.')
  }

  try {
    return await createHighlighterCore({
      langs,
      themes,
      engine,
    })
  }
  catch (error) {
    console.error('Failed to create highlighter:', error)
    throw error
  }
}

function parseColor(color?: string): string | undefined {
  if (!color)
    return undefined

  // Remove any whitespace
  color = color.trim()

  // Handle rgba/rgb format
  if (color.startsWith('rgb')) {
    return color
  }

  // Handle hex colors
  if (color.startsWith('#')) {
    // Convert 3-digit hex to 6-digit
    if (color.length === 4) {
      const r = color[1]
      const g = color[2]
      const b = color[3]
      color = `#${r}${r}${g}${g}${b}${b}`
    }

    // Handle 8-digit hex (with alpha)
    if (color.length === 9) {
      const r = Number.parseInt(color.slice(1, 3), 16)
      const g = Number.parseInt(color.slice(3, 5), 16)
      const b = Number.parseInt(color.slice(5, 7), 16)
      const a = Number.parseInt(color.slice(7, 9), 16) / 255
      return `rgba(${r}, ${g}, ${b}, ${a})`
    }

    // Handle 6-digit hex
    if (color.length === 7) {
      const r = Number.parseInt(color.slice(1, 3), 16)
      const g = Number.parseInt(color.slice(3, 5), 16)
      const b = Number.parseInt(color.slice(5, 7), 16)
      return `rgb(${r}, ${g}, ${b})`
    }
  }

  // If none of the above, return the original color
  return color
}

const SyntaxHighlighter: React.FC<SyntaxHighlighterProps> = ({ text, language, languageId, theme, fontSize = 14, scrollViewProps, textStyle, containerStyle }) => {
  const [tokens, setTokens] = useState<ThemedToken[][]>([])
  const [error, setError] = useState<Error | null>(null)

  const styles = useMemo(() => {
    const backgroundColor = parseColor(theme.colors?.['editor.background']) || '#1e1e1e'
    const foregroundColor = parseColor(theme.colors?.['editor.foreground']) || '#d4d4d4'

    console.log('[SyntaxHighlighter] Theme colors:', {
      background: backgroundColor,
      foreground: foregroundColor,
      allColors: theme.colors,
    })

    return StyleSheet.create({
      scrollView: {
        flex: 1,
        minHeight: 20,
      },
      codeContainer: {
        padding: 16,
        minWidth: '100%',
        backgroundColor: parseColor(theme.colors?.['editor.background']),
      },
      lineContainer: {
        flexDirection: 'row',
        flexWrap: 'wrap',
      },
      token: {
        fontFamily: FONT_FAMILY,
        color: parseColor(theme.colors?.['editor.foreground']),
      },
    })
  }, [theme])

  const baseStyle = useMemo(
    () => ({
      fontFamily: FONT_FAMILY,
      color: parseColor(theme.colors?.['editor.foreground']) || '#d8dee9',
    }),
    [theme],
  )

  // Highlight code when text/language changes
  useEffect(() => {
    let mounted = true

    async function highlight() {
      if (!text) {
        setTokens([])
        return
      }

      try {
        console.log('[SyntaxHighlighter] Creating highlighter for language:', languageId)
        const highlighter = await createHighlighter({
          langs: language ? [language] : [],
          themes: [Array.isArray(theme) ? theme[0] : theme],
        })

        console.log(
          '[SyntaxHighlighter] Theme token colors sample:',
          theme.tokenColors?.slice(0, 5).map(tc => ({
            scope: tc.scope,
            settings: {
              ...tc.settings,
              foreground: tc.settings.foreground,
              fontStyle: tc.settings.fontStyle,
            },
          })),
        )

        const result = highlighter.codeToTokens(text, {
          lang: languageId || 'plaintext',
          theme: Array.isArray(theme) ? theme[0] : theme,
        }) as TokensResult

        // And let's look at the token explanation too
        console.log(
          '[SyntaxHighlighter] First token explanation:',
          // @ts-expect-error - we know explanation exists because we asked for it
          result.tokens[0][0].explanation?.[0],
        )

        console.log('[SyntaxHighlighter] Result:', result)

        // Safely handle potential undefined tokens
        if (result.tokens && result.tokens.length > 0) {
          const firstLine = result.tokens[0]
          if (firstLine && firstLine.length > 0) {
            console.log(
              '[SyntaxHighlighter] First two tokens:',
              firstLine.slice(0, 2).map(token => ({
                content: token.content,
                color: token.color,
                explanation: token.explanation,
              })),
            )
          }
        }

        if (mounted && result.tokens) {
          setTokens(result.tokens)
          setError(null)
        }
      }
      catch (err) {
        console.error('Syntax highlighting failed:', err)
        if (mounted) {
          setError(err instanceof Error ? err : new Error('Highlighting failed'))
          // Fallback to plain text display on error
          setTokens([[{ content: text, color: styles.token.color, offset: 0 }]])
        }
      }
    }

    highlight()
    return () => {
      mounted = false
    }
  }, [text, language, languageId, theme])

  if (error) {
    // Fallback to plain text on error
    return (
      <ScrollView horizontal {...scrollViewProps} style={[styles.scrollView, scrollViewProps?.style]} contentContainerStyle={[styles.codeContainer, containerStyle]}>
        <Text style={[styles.token, { fontSize }, textStyle]}>{text}</Text>
      </ScrollView>
    )
  }

  interface TokenProps {
    token: ThemedToken
    fontSize: number
    textStyle?: StyleProp<TextStyle>
    baseStyle: TextStyle
  }

  const Token: React.FC<TokenProps> = React.memo(({ token, fontSize, textStyle, baseStyle }) => {
    // Create memoized token style
    const tokenStyle = useMemo(() => {
      const style: TextStyle = {
        color: parseColor(token.color),
        fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
        fontWeight: token.fontStyle === 1 ? '700' : '400',
        // Explicitly set other text properties to ensure they're not inherited incorrectly
        textDecorationLine: 'none',
        textAlign: 'left',
        fontSize,
      }
      return StyleSheet.flatten([baseStyle, style, textStyle])
    }, [token.color, token.fontStyle, fontSize, baseStyle, textStyle])

    if (__DEV__) {
      console.log('Token render:', {
        content: token.content,
        finalStyle: tokenStyle,
      })
    }

    return (
      <Text style={tokenStyle} selectable>
        {token.content}
      </Text>
    )
  })

  return (
    <ScrollView
      horizontal
      {...scrollViewProps}
      style={[styles.scrollView, scrollViewProps?.style]}
      contentContainerStyle={[styles.codeContainer, { backgroundColor: parseColor(theme.colors?.['editor.background']) || '#2e3440' }, containerStyle]}
    >
      <View>
        {tokens.map((line, lineIndex) => (
          <View key={lineIndex} style={styles.lineContainer}>
            {line.map((token, tokenIndex) => (
              <Token key={`${lineIndex}-${tokenIndex}`} token={token} fontSize={fontSize} textStyle={textStyle} baseStyle={baseStyle} />
            ))}
          </View>
        ))}
      </View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
